package patterns

// Java is a collection of common java related patterns.
var Java = map[string]string{
	"JAVACLASS": `(?:[a-zA-Z$_][a-zA-Z$_0-9]*\.)*[a-zA-Z$_][a-zA-Z$_0-9]*`,
	//Space is an allowed character to match special cases like 'Native Method' or 'Unknown Source'
	"JAVAFILE": `(?:[A-Za-z0-9_. -]+)`,
	//Allow special <init> method
	"JAVAMETHOD": `(?:(<init>)|[a-zA-Z$_][a-zA-Z$_0-9]*)`,
	//Line number is optional in special cases 'Native method' or 'Unknown source'
	"JAVASTACKTRACEPART": `%{SPACE}at %{JAVACLASS:class}\.%{JAVAMETHOD:method}\(%{JAVAFILE:file}(?::%{NUMBER:line})?\)`,
	// Java Logs
	"JAVATHREAD":             `(?:[A-Z]{2}-Processor[\d]+)`,
	"JAVACLASS_LOG":          `(?:[a-zA-Z0-9-]+\.)+[A-Za-z0-9$]+`,
	"JAVAFILE_LOG":           `(?:[A-Za-z0-9_.-]+)`,
	"JAVASTACKTRACEPART_LOG": `at %{JAVACLASS_LOG:class}\.%{WORD:method}\(%{JAVAFILE_LOG:file}:%{NUMBER:line}\)`,
	"JAVALOGMESSAGE":         `(.*)`,
	// MMM dd, yyyy HH:mm:ss eg: Jan 9, 2014 7:13:13 AM
	"CATALINA_DATESTAMP": `%{MONTH} %{MONTHDAY}, 20%{YEAR} %{HOUR}:?%{MINUTE}(?::?%{SECOND}) (?:AM|PM)`,
	// yyyy-MM-dd HH:mm:ss,SSS ZZZ eg: 2014-01-09 17:32:25,527 -0800
	"TOMCAT_DATESTAMP": `20%{YEAR}-%{MONTHNUM}-%{MONTHDAY} %{HOUR}:?%{MINUTE}(?::?%{SECOND}) %{ISO8601_TIMEZONE}`,
	"CATALINALOG":      `%{CATALINA_DATESTAMP:timestamp} %{JAVACLASS:class} %{JAVALOGMESSAGE:logmessage}`,
	// 2014-01-09 20:03:28,269 -0800 | ERROR | com.example.service.ExampleService - something compeletely unexpected happened...
	"TOMCATLOG": `%{TOMCAT_DATESTAMP:timestamp} \| %{LOGLEVEL:level} \| %{JAVACLASS:class} - %{JAVALOGMESSAGE:logmessage}`,
}
